using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SmartphoneSnake
{
    public partial class SnakeForm : Form
    {
        public SnakeForm()
        {
            init();
            InitializeComponent();
        }

        #region Snake Position Variables

        /// <summary>
        /// Diameter of a dot in the snake. Set at a 
        /// DotWidthFactorth of the display width 
        /// - rounded down
        /// </summary>
        private int dotDiameter;

        /// <summary>
        /// Size of the dots relative to the screen width.
        /// A value of n means that n dots would fit in the screen
        /// width. 
        /// </summary>
        private const int DotWidthFactor = 20;

        /// <summary>
        /// X coordinate of top left hand corner of the
        /// snake head. Origin at left.
        /// </summary>
        private float snakeX;

        /// <summary>
        /// Y coordinate of top left hand corner of the 
        /// snake head. Origin at top.
        /// </summary>
        private float snakeY;

        #endregion 

        #region Snake Drawing variables

        /// <summary>
        /// Brush used to draw the snake body. Kept as a member
        /// to avoid repeated recreation. Created in init.
        /// </summary>
        private Brush snakeBrush;

        /// <summary>
        /// Graphics device for this form. Used to draw all
        /// screen items. Assigned in init.
        /// </summary>
        private Graphics gameGraphics;

        #endregion

        #region Game State variables

        private bool gameActive = false;

        #endregion

        /// <summary>
        /// Sets up the game at the start.
        /// </summary>
        private void init()
        {
            snakeBrush = new SolidBrush(Color.Red);

            gameGraphics = this.CreateGraphics();

            dotDiameter = this.ClientRectangle.Width / DotWidthFactor;

            snakeX = (this.ClientRectangle.Width - dotDiameter) / 2;
            snakeY = (this.ClientRectangle.Height - dotDiameter) / 2;
        }

        /// <summary>
        /// draw the game screen using the graphics for this form.
        /// </summary>
        private void drawGameScreen()
        {
            gameGraphics.FillEllipse(snakeBrush,
                (int)snakeX, (int)snakeY,
                dotDiameter, dotDiameter);
        }

        #region Game Play 

        /// <summary>
        /// Plays the game. Sets gameActive to true and
        /// updates the game until it becomes false.
        /// </summary>
        private void playGame()
        {
            gameActive = true;
            while (gameActive)
            {
                drawGameScreen();
            }
        }

        /// <summary>
        /// Refers to a method with the signature required for 
        /// Form.Invoke calls.
        /// </summary>
        private delegate void gameMethod();

        /// <summary>
        /// Refers to the method which will play the game. Set
        /// to playGame when the game starts.
        /// </summary>
        gameMethod playDelegate;

        /// <summary>
        /// Starts a new game.
        /// </summary>
        private void startGame()
        {
            playDelegate = new gameMethod(playGame);
            Invoke(playDelegate);
        }


        #endregion 

        #region Event Handlers

        protected override void OnPaint(PaintEventArgs e)
        {
            drawGameScreen();
        }

        private void playMenuItem_Click(object sender, EventArgs e)
        {
            startGame();
        }

        #endregion
    }
}