using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SmartphoneSnake
{
    public partial class SnakeForm : Form
    {
        public SnakeForm()
        {
            init();
            InitializeComponent();
        }

        #region Snake Position Variables

        /// <summary>
        /// Diameter of a dot in the snake. Set at a 
        /// DotWidthFactorth of the display width 
        /// - rounded down
        /// </summary>
        private int dotDiameter;

        /// <summary>
        /// Size of the dots relative to the screen width.
        /// A value of n means that n dots would fit in the screen
        /// width. 
        /// </summary>
        private const int DotWidthFactor = 20;

        /// <summary>
        /// X coordinate of top left hand corner of the
        /// snake head. Origin at left.
        /// </summary>
        private float snakeX;

        /// <summary>
        /// Y coordinate of top left hand corner of the 
        /// snake head. Origin at top.
        /// </summary>

        private float snakeY;
        /// <summary>
        /// Speed of snake in x direction.
        /// </summary>
        private float snakeXSpeed;

        /// <summary>
        /// Speed of snake in y direction.
        /// </summary>
        private float snakeYSpeed;

        /// <summary>
        /// The start speed of the snake.
        /// </summary>
        private float initialSnakeSpeed;

        /// <summary>
        /// The maximum speed of the snake.
        /// </summary>
        private float maximumSnakeSpeed;

        /// <summary>
        /// The change in speed that is applied to the snake.
        /// </summary>
        private float snakeSpeedStep;

        /// <summary>
        /// The speed of movement of the snake itself.
        /// </summary>
        private float snakeSpeed;

        /// <summary>
        /// Limit for the snake X value. Width of the screen
        /// minus the diameter of the snake dot.
        /// </summary>
        private float snakeXLimit;

        /// <summary>
        /// Limit for the snake Y value. Height of the screen 
        /// minus the diameter of the snake dot.
        /// </summary>
        private float snakeYLimit;

        #endregion 

        #region Snake Drawing variables

        /// <summary>
        /// Brush used to draw the snake body. Kept as a member
        /// to avoid repeated recreation. Created in init.
        /// </summary>
        private Brush snakeBrush;

        /// <summary>
        /// Brush used to draw the background. Kept as a member
        /// to avoid repeated recreation. Created in init.
        /// </summary>
        private Brush backgroundBrush;

        /// <summary>
        /// Graphics device for this form. Used to draw all
        /// screen items. Assigned in init.
        /// </summary>
        private Graphics gameGraphics;

        #endregion

        #region Game State variables

        private bool gameActive = false;

        #endregion

        /// <summary>
        /// Sets up the game at the start.
        /// </summary>
        private void init()
        {
            snakeBrush = new SolidBrush(Color.Red);
            backgroundBrush = new SolidBrush(Color.White);

            gameGraphics = this.CreateGraphics();

            dotDiameter = this.ClientRectangle.Width / DotWidthFactor;

            snakeX = (this.ClientRectangle.Width - dotDiameter) / 2;
            snakeY = (this.ClientRectangle.Height - dotDiameter) / 2;

            initialSnakeSpeed = (float)dotDiameter / 5.0f;
            maximumSnakeSpeed = (float)dotDiameter / 2.0f;
            snakeSpeedStep = (maximumSnakeSpeed - initialSnakeSpeed) / 5;

            snakeSpeed = initialSnakeSpeed;

            snakeXSpeed = snakeSpeed;
            snakeYSpeed = 0;

            snakeXLimit = this.ClientRectangle.Width - dotDiameter;
            snakeYLimit = this.ClientRectangle.Height - dotDiameter;

        }

        /// <summary>
        /// Update the snake position. End the game if
        /// it hits anything.
        /// </summary>
        private void updateSnake()
        {
            snakeX += snakeXSpeed;
            snakeY += snakeYSpeed;
            if ((snakeX < 0.0f) || (snakeX > snakeXLimit) ||
                 (snakeY < 0.0f) || (snakeY > snakeYLimit))
            {
                MessageBox.Show("Game Over");
                gameActive = false;
            }
        }

        /// <summary>
        /// draw the game screen using the graphics for this form.
        /// </summary>
        private void drawGameScreen()
        {
            gameGraphics.FillEllipse(snakeBrush,
                (int)snakeX, (int)snakeY,
                dotDiameter, dotDiameter);
        }

        /// <summary>
        /// Erases the game screen using the graphics for this form.
        /// </summary>
        private void eraseGameScreen()
        {
            gameGraphics.FillEllipse(backgroundBrush,
                (int)snakeX, (int)snakeY,
                dotDiameter, dotDiameter);
        }

        #region Game Play

        /// <summary>
        /// Plays the game. Sets gameActive to true and
        /// updates the game until it becomes false.
        /// </summary>
        public void playGame()
        {
            init();
            gameActive = true;
            while (gameActive)
            {
                eraseGameScreen();
                updateSnake();
                drawGameScreen();
                System.Threading.Thread.Sleep(50);
                Application.DoEvents();
            }
        }

        /// <summary>
        /// Refers to a method with the signature required for 
        /// Form.Invoke calls.
        /// </summary>
        private delegate void gameMethod();

        /// <summary>
        /// Refers to the method which will play the game. Set
        /// to playGame when the game starts.
        /// </summary>
        gameMethod playDelegate;

        /// <summary>
        /// Starts a new game.
        /// </summary>
        private void startGame()
        {
            playDelegate = new gameMethod(playGame);
            Invoke(playDelegate);
        }

        #endregion 

        #region Event Handlers

        protected override void OnPaint(PaintEventArgs e)
        {
            drawGameScreen();
        }

        private void playMenuItem_Click(object sender, EventArgs e)
        {
            playGame();
        }

        private void SnakeForm_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.F1))
            {
                // Soft Key 1
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F2))
            {
                // Soft Key 2
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Up))
            {
                // Up
                snakeXSpeed = 0.0f;
                snakeYSpeed = -snakeSpeed;
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Down))
            {
                // Down
                snakeXSpeed = 0.0f;
                snakeYSpeed = snakeSpeed;
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Left))
            {
                // Left
                snakeXSpeed = -snakeSpeed;
                snakeYSpeed = 0.0f;
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Right))
            {
                // Right
                snakeXSpeed = snakeSpeed;
                snakeYSpeed = 0.0f;
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
            {
                // Enter
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D1))
            {
                // 1
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D2))
            {
                // 2
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D3))
            {
                // 3
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D4))
            {
                // 4
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D5))
            {
                // 5
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D6))
            {
                // 6
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D7))
            {
                // 7
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D8))
            {
                // 8
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D9))
            {
                // 9
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F8))
            {
                // *
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D0))
            {
                // 0
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F9))
            {
                // #
            }
        }
        #endregion
    }
}